messages = {
    "GENERIC_CREATION_SUCCESSFUL": "records created successfully.",
    "GENERIC_READ_SUCCESSFUL": "data retrieved successfully.",
    "GENERIC_UPDATE_SUCCESSFUL": "your information has been updated successfully.",
    "GENERIC_DELETE_SUCCESSFUL": "your records have been deleted successfully.",
    "GENERIC_400": "the request is invalid or cannot be processed.",
    "GENERIC_500": "an internal server error occurred. please try again later.",
}
